<?xml version="1.0" encoding="UTF-8"?>
<gui name="%{APPNAMELC}"
     version="1"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">
<MenuBar>
  <Menu name="edit">
    <Action name="switch_action"/>
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar" noMerge="1">
    <text>Main Toolbar</text>
    <Action name="switch_action"/>
    <Separator/>
    <Action name="options_configure"/>
</ToolBar>
</gui>
